#pragma once

#include <thrust/host_vector.h>
#include <thrust/device_vector.h>

constexpr size_t VECTOR_SIZE = 100000000;	// Number of elements to compute

class CudaWorker
{
public:
	CudaWorker();
	~CudaWorker();

	// Disable special member functions
	CudaWorker(const CudaWorker& other) = delete;
	CudaWorker(CudaWorker&& other) noexcept = delete;
	CudaWorker& operator=(const CudaWorker& other) = delete;
	CudaWorker& operator=(CudaWorker&& other) = delete;

	// Perform the computation using a single thread on the CPU
	void CpuCompute();

	// Perform the computation using the GPU in parallel
	void GpuCompute();

	// Verify that the answers match between the CPU and GPU
	bool Verify();

private:
	// Storage on the host
	thrust::host_vector<float> a;				// Values for one one side of the triangle
	thrust::host_vector<float> b;				// Values for the other side of the triangle
	thrust::host_vector<float> cpu_result;		// Result values from CPU computation
	thrust::host_vector<float> gpu_result;		// Result values from GPU computation

	// Storgage on the gpu
	thrust::device_vector<float> device_a;
	thrust::device_vector<float> device_b;
	thrust::device_vector<float> device_result;
};