﻿#include <chrono>
#include <iostream>

using namespace std;
using std::chrono::high_resolution_clock;
using std::chrono::duration_cast;
using std::chrono::duration;
using std::chrono::microseconds;

#include "CudaWorker.cuh"
#include "Gpu.h"

int main()
{
    Gpu gpu;            // Singleton to mange the GPU, initializes the first card
    CudaWorker worker;  // A instance of the work we want to accomplish

    cout << endl;

    // Perform the work on the CPU and report the time
    auto start_cpu_time = high_resolution_clock::now();
    worker.CpuCompute();
    auto end_cpu_time = high_resolution_clock::now();
    cout << "CpuCompute took " << duration_cast<microseconds>(end_cpu_time - start_cpu_time).count() << "us" << endl;

    // Perform the work on the GPU and report the time
    auto start_gpu_time = high_resolution_clock::now();
    worker.GpuCompute();
    auto end_gpu_time = high_resolution_clock::now();
    cout << "GpuCompute took " << duration_cast<microseconds>(end_gpu_time - start_gpu_time).count() << "us" << endl;

    // Make sure the answers match between the CPU and GPU
    if (worker.Verify()) {
        cout << "Answers match!" << endl;
    }
    else {
        cout << "Answers DO NOT match!" << endl;
    }

    return 0;
}
